;;;
;;; Theme source for "Latem" theme.
;;;

(let*
    ; This is the font gtk+ metal uses.
    ((font (get-font "-b&h-lucida-bold-r-normal-sans-12-*-*-*-p-*-iso8859-1"))
     (font-colors         '(list (get-color "#000000")
				 (get-color "#000000")))

     ; Shamelessly ripped from Moondome
     (update-title-width
      (lambda (w)
        (if (eq (window-get w 'current-frame-style) 'Latem)
	    (rebuild-frame w))))


     ;; The titlebar
     (lside-width          21)      ; Before the title text
     (title-text-space     3)       ; Between button and text
     (rside-width          65)      ; All the rest
     
     (title-colors         (list (get-color "#cccccc")
				 (get-color "#ccccff")))
     (title-left-images    (list (make-image "title-left-inactive.png")
				 (make-image "title-left-active.png")))
     (menu-button-images   (list (make-image "menu-button.png")
				 (make-image "menu-button.png")))

     (title-text-width
      (lambda (w)
	(let ((win-width (car (window-dimensions w)))
	      (tw        (text-width (window-name w) font)))
	  ; stop the texture appearing to shift when it changes size
	  (setq tw (+ tw (- 4 (mod tw 4))))  
	  (cond
	   ; If all our basic widths are less than the total width, OK
	   ((< (+ lside-width title-text-space rside-width tw)
	       win-width)
	    (+ title-text-space tw))
	   ; If all widths minus left texture space (and thus the texture), OK
	   ((> (+ lside-width title-text-space rside-width tw)
	       win-width)
	    (+ title-text-space tw))
	   ; Otherwise we have to clip
	   (t (- win-width lside-width title-text-space rside-width))))))

     (title-texture-width
      (lambda (w)
	(let ((win-width (car (window-dimensions w)))
	      (tw        (title-text-width w)))
	  (cond
	   ; Space left over? Put some texture in.
	   ((> win-width (+ tw lside-width rside-width))
	    (- win-width (+ lside-width title-text-space tw rside-width)))
	   ; Just put none in.
	   (t 0)))))
     
     (title-texture-images
      (let ((img1 (make-image "title-texture-inactive.png"))
	    (img2 (make-image "title-texture-active.png")))
	(image-put img1 'tiled t)
	(image-put img2 'tiled t)
	(list img1 img2)))
     
     (min-button-images    (list (make-image "min-button-inactive.png")
				 (make-image "min-button-active.png")
				 (make-image "min-button-active.png")
				 (make-image "min-button-active-clicked.png")))
     (max-button-images    (list (make-image "max-button-inactive.png")
				 (make-image "max-button-active.png")
				 (make-image "max-button-active.png")
				 (make-image "max-button-active-clicked.png")))
     (unmax-button-images  (list (make-image "unmax-button-inactive.png")
				 (make-image "unmax-button-active.png")
				 (make-image "unmax-button-active.png")
				 (make-image "unmax-button-active-clicked.png")))
     (max-unmax-button-images (lambda (w)
				(if (window-maximized-p w)
				    unmax-button-images
				  max-button-images)))
     (close-button-images  (list (make-image "close-button-inactive.png")
				 (make-image "close-button-active.png")
				 (make-image "close-button-active.png")
				 (make-image "close-button-active-clicked.png")))
     (title-right-images   (list (make-image "title-right-inactive.png")
				 (make-image "title-right-active.png")))

     ; That Little Bar
     (little-bar-colors    (list (get-color "#666699")
				 (get-color "#666666")))
     
     ; Borders
     (horz-border-left     (list (make-image "horz-border-inactive-left.png")
				 (make-image "horz-border-active-left.png")))
     (horz-border-mid      (list (make-image "horz-border-inactive-mid.png")
				 (make-image "horz-border-active-mid.png")))
     (horz-border-right    (list (make-image "horz-border-inactive-right.png")
				 (make-image "horz-border-active-right.png")))
     (vert-border-top      (list (make-image "vert-border-inactive-top.png")
				 (make-image "vert-border-active-top.png")))
     (vert-border-mid      (list (make-image "vert-border-inactive-mid.png")
				 (make-image "vert-border-active-mid.png")))
     (vert-border-bot      (list (make-image "vert-border-inactive-bot.png")
				 (make-image "vert-border-active-bot.png")))

     ; Finally, we put it all together
     (normal-frame
      `(((class       . title)
	 (background  . ,title-left-images)
	 (top-edge    . -21)
	 (left-edge   . 0))
	((class       . menu-button)
	 (background  . ,title-colors)
	 (foreground  . ,menu-button-images)
	 (top-edge    . -21)
	 (left-edge   . 5)
	 (width       . 16)
	 (height      . 20)
	 (y-justify   . center))
	((class       . title)
	 (background  . ,title-colors)
	 (foreground  . ,font-colors)
	 (font        . ,font)
	 (text        . ,window-name)
	 (top-edge    . -21)
	 (left-edge   . 21)
	 (width       . ,title-text-width)
 	 (height      . 20)
	 (x-justify   . ,title-text-space)
	 (y-justify   . center))
	((class       . title)
	 (background  . ,title-colors)
	 (top-edge    . -21)
	 (left-edge   . ,(lambda (w) (+ lside-width (title-text-width w))))
	 (width       . 3)
	 (height      . 20))
	((class       . title)
	 (background  . ,title-texture-images)
	 (top-edge    . -21)
	 (left-edge   . ,(lambda (w) (+ lside-width 3 (title-text-width w))))
	 (width       . ,title-texture-width)
	 (height      . 20))
	((class       . iconify-button)
	 (background  . ,title-colors)
	 (foreground  . ,min-button-images)
	 (top-edge    . -21)
	 (right-edge  . 46)
	 (width       . 19)
	 (height      . 20)
	 (x-justify   . 3))
	((class       . maximize-button)
	 (background  . ,title-colors)
	 (foreground  . ,max-unmax-button-images)
	 (top-edge    . -21)
	 (right-edge  . 23)
	 (width       . 23)
	 (height      . 20)
	 (x-justify   . 2))
	((class       . close-button)
	 (background  . ,title-colors)
	 (foreground  . ,close-button-images)
	 (top-edge    . -21)
	 (right-edge  . 4)
	 (width       . 20)
	 (height      . 20)
	 (x-justify   . 5))
	((class       . close-button)
	 (background  . ,title-right-images)
	 (top-edge    . -21)
	 (right-edge  . 0)
	 (width       . 4)
	 (height      . 20))
	((class       . titlebar)
	 (background  . ,little-bar-colors)
	 (top-edge    . -1)
	 (left-edge   . 0)
	 (width       . ,(lambda (w) (car (window-dimensions w))))
	 (height      . 1))
	
	;; OK, this is the border!
	; Top
	((class       . top-left-corner)
	 (background  . ,horz-border-left)
	 (top-edge    . -26)
	 (left-edge   . -4)
	 (width       . 15)
	 (height      . 5))
	((class       . top-border)
	 (background  . ,horz-border-mid)
	 (top-edge    . -26)
	 (left-edge   . 11)
	 (width       . ,(lambda (w) (max 0 (- (car (window-dimensions w)) 22))))
	 (height      . 5))
	((class       . top-right-corner)
	 (background  . ,horz-border-right)
	 (top-edge    . -26)
	 (right-edge  . -4)
	 (width       . 15)
	 (height      . 5))
	; Left
	((class       . top-left-corner)
	 (background  . ,vert-border-top)
	 (top-edge    . -25)
	 (left-edge   . -5)
	 (width       . 5)
	 (height      . 15))
	((class       . left-border)
	 (background  . ,vert-border-mid)
	 (top-edge    . -10)
	 (left-edge   . -5)
	 (width       . 5)
	 (height      . ,(lambda (w) (cdr (window-dimensions w))))) ; +10-10
	((class       . bottom-left-corner)
	 (background  . ,vert-border-bot)
	 (bottom-edge . -4)
	 (left-edge   . -5)
	 (width       . 5)
	 (height      . 15))
	; Right
	((class       . top-right-corner)
	 (background  . ,vert-border-top)
	 (top-edge    . -25)
	 (right-edge  . -5)
	 (width       . 5)
	 (height      . 15))
	((class       . right-border)
	 (background  . ,vert-border-mid)
	 (top-edge    . -10)
	 (right-edge   . -5)
	 (width       . 5)
	 (height      . ,(lambda (w) (cdr (window-dimensions w))))) ; +10-10
	((class       . bottom-right-corner)
	 (background  . ,vert-border-bot)
	 (bottom-edge . -4)
	 (right-edge   . -5)
	 (width       . 5)
	 (height      . 15))
	; Bottom
	((class       . bottom-left-corner)
	 (background  . ,horz-border-left)
	 (bottom-edge . -5)
	 (left-edge   . -4)
	 (width       . 15)
	 (height      . 5))
	((class       . bottom-border)
	 (background  . ,horz-border-mid)
	 (bottom-edge . -5)
	 (left-edge   . 11)
	 (width       . ,(lambda (w) (max 0 (- (car (window-dimensions w)) 22))))
	 (height      . 5))
	((class       . bottom-right-corner)
	 (background  . ,horz-border-right)
	 (bottom-edge . -5)
	 (right-edge  . -4)
	 (width       . 15)
	 (height      . 5))))

     ; No borders for shape windows
     (shaped-frame
      `(((class       . title)
	 (background  . ,title-left-images)
	 (top-edge    . -21)
	 (left-edge   . 0))
	((class       . menu-button)
	 (background  . ,title-colors)
	 (foreground  . ,menu-button-images)
	 (top-edge    . -21)
	 (left-edge   . 5)
	 (width       . 16)
	 (height      . 20)
	 (y-justify   . center))
	((class       . title)
	 (background  . ,title-colors)
	 (foreground  . ,font-colors)
	 (text        . ,window-name)
	 (top-edge    . -21)
	 (left-edge   . 21)
	 (width       . ,title-text-width)
 	 (height      . 20)
	 (x-justify   . ,title-text-space)
	 (y-justify   . center))
	((class       . title)
	 (background  . ,title-colors)
	 (top-edge    . -21)
	 (left-edge   . ,(lambda (w) (+ lside-width (title-text-width w))))
	 (width       . 3)
	 (height      . 20))
	((class       . title)
	 (background  . ,title-texture-images)
	 (top-edge    . -21)
	 (left-edge   . ,(lambda (w) (+ lside-width 3 (title-text-width w))))
	 (width       . ,title-texture-width)
	 (height      . 20))
	((class       . iconify-button)
	 (background  . ,title-colors)
	 (foreground  . ,min-button-images)
	 (top-edge    . -21)
	 (right-edge  . 46)
	 (width       . 19)
	 (height      . 20)
	 (x-justify   . 3))
	((class       . maximize-button)
	 (background  . ,title-colors)
	 (foreground  . ,max-unmax-button-images)
	 (top-edge    . -21)
	 (right-edge  . 23)
	 (width       . 23)
	 (height      . 20)
	 (x-justify   . 2))
	((class       . close-button)
	 (background  . ,title-colors)
	 (foreground  . ,close-button-images)
	 (top-edge    . -21)
	 (right-edge  . 4)
	 (width       . 20)
	 (height      . 20)
	 (x-justify   . 5))
	((class       . close-button)
	 (background  . ,title-right-images)
	 (top-edge    . -21)
	 (right-edge  . 0)
	 (width       . 4)
	 (height      . 20))
	((class       . titlebar)
	 (background  . ,little-bar-colors)
	 (top-edge    . -1)
	 (left-edge   . 0)
	 (width       . ,(lambda (w) (car (window-dimensions w))))
	 (height      . 1))))

     (border-only-frame
        ; Top
      `(((class       . top-left-corner)
	 (background  . ,horz-border-left)
	 (top-edge    . -5)
	 (left-edge   . -4)
	 (width       . 15)
	 (height      . 5))
	((class       . top-border)
	 (background  . ,horz-border-mid)
	 (top-edge    . -5)
	 (left-edge   . 11)
	 (width       . ,(lambda (w) (max 0 (- (car (window-dimensions w)) 22))))
	 (height      . 5))
	((class       . top-right-corner)
	 (background  . ,horz-border-right)
	 (top-edge    . -5)
	 (right-edge  . -4)
	 (width       . 15)
	 (height      . 5))
	; Left
	((class       . top-left-corner)
	 (background  . ,vert-border-top)
	 (top-edge    . -4)
	 (left-edge   . -5)
	 (width       . 5)
	 (height      . 15))
	((class       . left-border)
	 (background  . ,vert-border-mid)
	 (top-edge    . 11)
	 (left-edge   . -5)
	 (width       . 5)
	 (height      . ,(lambda (w) (cdr (window-dimensions w))))) ; +10-10
	((class       . bottom-left-corner)
	 (background  . ,vert-border-bot)
	 (bottom-edge . -4)
	 (left-edge   . -5)
	 (width       . 5)
	 (height      . 15))
	; Right
	((class       . top-right-corner)
	 (background  . ,vert-border-top)
	 (top-edge    . -4)
	 (right-edge  . -5)
	 (width       . 5)
	 (height      . 15))
	((class       . right-border)
	 (background  . ,vert-border-mid)
	 (top-edge    . 11)
	 (right-edge   . -5)
	 (width       . 5)
	 (height      . ,(lambda (w) (cdr (window-dimensions w))))) ; +10-10
	((class       . bottom-right-corner)
	 (background  . ,vert-border-bot)
	 (bottom-edge . -4)
	 (right-edge  . -5)
	 (width       . 5)
	 (height      . 15))
	; Bottom
	((class       . bottom-left-corner)
	 (background  . ,horz-border-left)
	 (bottom-edge . -5)
	 (left-edge   . -4)
	 (width       . 15)
	 (height      . 5))
	((class       . bottom-border)
	 (background  . ,horz-border-mid)
	 (bottom-edge . -5)
	 (left-edge   . 11)
	 (width       . ,(lambda (w) (max 0 (- (car (window-dimensions w)) 22))))
	 (height      . 5))
	((class       . bottom-right-corner)
	 (background  . ,horz-border-right)
	 (bottom-edge . -5)
	 (right-edge  . -4)
	 (width       . 15)
	 (height      . 5)))))
  
     (add-frame-style 'Latem
		      (lambda (w type)
			(cond
			 ((eq type 'shaped)    shaped-frame)
			 ((eq type 'transient) border-only-frame)
			 ((eq type 'unframed)  nil-frame)
			 ((eq type 'default)   normal-frame)
                         (t nil))))

     (call-after-property-changed 'WM_NAME update-title-width))
